from os import system, name

from termcolor import colored
import pika
from decouple import config


RABBIT_MQ_HOST = config('RABBIT_MQ_HOST')
RABBIT_MQ_PORT = config('RABBIT_MQ_PORT')


class Utils:
    

    def printStart():
        programName = """
               _____ _____ _  ___   _ ___ ____    _      ___    _    
              |_   _| ____| |/ | \ | |_ _/ ___|  / \    |_ _|  / \   
                | | |  _| | ' /|  \| || |\___ \ / _ \    | |  / _ \  
                | | | |___| . \| |\  || | ___) / ___ \   | | / ___ \ 
                |_| |_____|_|\_|_| \_|___|____/_/   \_\ |___/_/   \_\ 
        """

        print(colored(programName, 'green'))

    def clearConsole():  
        # for windows 
        if name == 'nt': 
            _ = system('cls') 

        # for mac and linux(here, os.name is 'posix') 
        else: 
            _ = system('clear')

    def send_message_rabbitmq(rabbitmq_queue, message):

        print("Enviando mensagem")
        print(message)

        global RABBIT_MQ_HOST
        global RABBIT_MQ_PORT

        connection = pika.BlockingConnection(pika.ConnectionParameters(host = RABBIT_MQ_HOST, port = RABBIT_MQ_PORT))
        channel = connection.channel()

        channel.queue_declare(queue = rabbitmq_queue)

        channel.basic_publish(exchange = '', routing_key = rabbitmq_queue, body = message)
        connection.close()


import threading
import requests 
import os
import traceback

from flask import Flask, render_template, Response, request, make_response, current_app, jsonify
import cv2
from decouple import config
from datetime import datetime, timedelta
from functools import update_wrapper

from classes.camera import VideoCamera
from classes.encode_faces import EncodeFace

app = Flask(__name__)

HTTP_HOST = config('HTTP_HOST')
HTTP_PORT = config('HTTP_PORT')

camera = None

class WebServer:

    def __init__(self, camera_class):
        global camera
        camera = camera_class


    def crossdomain(origin=None, methods=None, headers=None, max_age=21600,
                attach_to_all=True, automatic_options=True):
        """Decorator function that allows crossdomain requests.
            Courtesy of
            https://blog.skyred.fi/articles/better-crossdomain-snippet-for-flask.html
        """
        if methods is not None:
            methods = ', '.join(sorted(x.upper() for x in methods))
        # use str instead of basestring if using Python 3.x
        if headers is not None and not isinstance(headers, basestring):
            headers = ', '.join(x.upper() for x in headers)
        # use str instead of basestring if using Python 3.x
        if not isinstance(origin, str):
            origin = ', '.join(origin)
        if isinstance(max_age, timedelta):
            max_age = max_age.total_seconds()

        def get_methods():
            """ Determines which methods are allowed
            """
            if methods is not None:
                return methods

            options_resp = current_app.make_default_options_response()
            return options_resp.headers['allow']

        def decorator(f):
            """The decorator function
            """
            def wrapped_function(*args, **kwargs):
                """Caries out the actual cross domain code
                """
                if automatic_options and request.method == 'OPTIONS':
                    resp = current_app.make_default_options_response()
                else:
                    resp = make_response(f(*args, **kwargs))
                if not attach_to_all and request.method != 'OPTIONS':
                    return resp

                h = resp.headers
                h['Access-Control-Allow-Origin'] = origin
                h['Access-Control-Allow-Methods'] = get_methods()
                h['Access-Control-Max-Age'] = str(max_age)
                h['Access-Control-Allow-Credentials'] = 'true'
                h['Access-Control-Allow-Headers'] = \
                    "Origin, X-Requested-With, Content-Type, Accept, Authorization"
                if headers is not None:
                    h['Access-Control-Allow-Headers'] = headers
                return resp

            f.provide_automatic_options = False
            return update_wrapper(wrapped_function, f)
        return decorator
    

    def startServer(self):
        global HTTP_HOST
        global HTTP_PORT
        print(" [*] Starting WebServer.")
        app.run(host=HTTP_HOST, debug=False, port=HTTP_PORT)

    @app.route('/video_feed', methods=['GET', 'OPTIONS'])
    def video_feed():

        global camera

        def gen(camera_videofeed):
            try:
                while True:
                    frame, _ = camera_videofeed.get_frame()
                    yield (b'--frame\r\n'
                        b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n\r\n')
            except Exception as e:
                print(str(e))
                print(traceback.format_exc())

        return Response(gen(camera),
                        mimetype='multipart/x-mixed-replace; boundary=frame')
    
    @app.route('/mode/<mode>', methods=['GET', 'OPTIONS'])
    @crossdomain(origin='*')
    def set_mode(mode):
        global camera

        # modes: register and detect

        camera.mode = mode

        return jsonify({"status": True})

    @app.route('/getcod/<cpf>', methods=['GET', 'OPTIONS'])
    @crossdomain(origin='*')
    def get_cpf(cpf):
        result_response = {
            "status": True,
            "message": "Operação realizada com sucesso."
        }

        try:

            data = {
                "CPF": cpf
            }

            r = requests.post(url = 'http://briansilva1.zeedhi.com/workfolder/integracao-catraca/backend/service/index.php/getcodbycpf', data = data)
            result = r.json() 
            if(result['status']):
                result_response['CDFUNC'] = result['CDFUNC']        
            else:
                raise Exception(result['mensagem'])
        
        except Exception as e:
            result_response['status'] = False
            result_response['message'] = str(e)


        return jsonify(result_response)

    


    @app.route('/takepersonphoto/<person_cpf>', methods=['GET', 'OPTIONS'])
    @crossdomain(origin='*')
    def register_person(person_cpf):
        global camera

        result_response = {
            "status": True,
            "message": "Operação realizada com sucesso."
        }

        try:

            if (camera.mode != 'register'):
                raise Exception('Modo de registro não está ativo')

            data = {
                "CPF": person_cpf
            }

            # sending get request and saving the response as response object
            r = requests.post(url = 'http://briansilva1.zeedhi.com/workfolder/integracao-catraca/backend/service/index.php/getcodbycpf', data = data)

            result = r.json()

            if (result['status']):
                    
                person_cod = result['CDFUNC']

                path = './face_recognition/dataset/' + str(person_cod)
                if os.path.exists(path) == False:
                    os.makedirs(path)          
                _, frame = camera.get_frame()
                date = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
                img_name = "/"+date+".jpg"
                cv2.imwrite(path+img_name,frame)
            
            else:
                raise Exception(result['message'])
        
        except Exception as e:
            result_response['status'] = False
            result_response['message'] = str(e)
            print(str(e))
            print(traceback.format_exc())

        return jsonify(result_response)

    @app.route('/startTraining', methods=['GET', 'OPTIONS'])
    @crossdomain(origin='*')
    def start_training():
        encodeFace = EncodeFace()
        response = encodeFace.enconde()        
        return jsonify(response)



    def main(self):    
        t1 = threading.Thread(target=self.startServer, daemon=True )
        t1.start()


import time
import threading
import random
import requests
from decouple import config
import json

from classes.person import Person


class PersonManager():

    def __init__(self):       
        self.person = []

    def get_person(self):
        r = requests.get(config("GET_ORDERS")+"orders") 
        data  = r.json()
        order_list = data['items']
        return order_list

    def get_person_name(person_cod):

        headers = {'content-type': 'application/json'}
        data = {
            "CDFUNC": person_cod 
        }        
        r = requests.post(config("API_ADDRESS")+"operador",headers=headers, data=json.dumps(data))
        data  = r.json()
        return data["NMFUNC"]
  


import cv2
import uuid
import time
import threading

import os
import json
from decouple import config

current_milli_time = lambda: int(round(time.time() * 1000))
TEMPO_TOLERANCIA_ROSTO = 0.5
PATH_TMP_IMGS = 'tmpimgs/'
IMAGE_NAME = "tmpimg.jpg"
NUM_CAMERA=config("NUM_CAMERA")

class VideoCamera():
    
    def __init__(self):
        # Using OpenCV to capture from device 0. If you have trouble capturing
        # from a webcam, comment the line below out and use a video file
        # instead.
        self.video = cv2.VideoCapture(int(NUM_CAMERA))
        self.possuiFace = False
        self.oldFaces = []
        self.imagemJpeg = None
        self.image = self.video.read()
        self.available = False
        self.mode = "detect"

        self.ultimaRequisicao = time.time()

    def __del__(self):
        self.video.release()
        
    def desenharRosto(self, image, x, y, w, h, blur = False):
        # To draw a rectangle in a face  
        cv2.rectangle(image, (x,y), (x+w,y+h), (255,255,0) ,2)

        if (blur):
            result_image = cv2.blur(image, (7, 7))
            sub_face = image[y:y+h, x:x+w]
            result_image[y:y+sub_face.shape[0], x:x+sub_face.shape[1]] = sub_face
            image = result_image

        return image

    def get_frame(self):
        self.success, self.image = self.video.read()
        self.available = True

        # We are using Motion JPEG,  but OpenCV defaults to capture raw images,
        # so we must encode it into JPEG in order to correctly display the
        # video stream.
        ret, imagemJpeg = cv2.imencode('.jpg', self.image)

        self.imagemJpeg = imagemJpeg.tobytes()

        return (self.imagemJpeg, self.image)
    
    def capturarFoto(self):
        i = 1

        with os.scandir(PATH_TMP_IMGS) as entries:
            for entry in entries:
                if entry.is_file() or entry.is_symlink():
                    os.remove(entry.path)

        for (x,y,w,h) in self.oldFaces:
            roi_color = self.image[y:y + h, x:x + w]
            cv2.imwrite(PATH_TMP_IMGS+str(i)+ '_tmpface.jpg', roi_color)
            i=i+1

        cv2.imwrite(IMAGE_NAME, self.image)

    def analisarFace(self):
        face_cascade = cv2.CascadeClassifier('haarcascade_frontalface_default.xml') 
        gray = cv2.cvtColor(self.image, cv2.COLOR_BGR2GRAY) 
        faces = face_cascade.detectMultiScale(gray, 1.3, 5) 
        
        # Verifica se alguma face foi detectada
        # Caso não tenha sido, verifica se tem X segundos que nenhuma face eh detectada
        # Caso tenha mais tempo que o tempo de tolerancia, marca como nenhuma face
        # Caso tenha menos tempo que o tempo de tolerancia desenha o ultimo quadrado desenhado
        if(len(faces) > 0):
            # self.ws.enviarMensagem("Possui face.")

            self.possuiFace = True
            self.momentoUltimaFaceDetectada = current_milli_time()
            self.oldFaces = faces
     

            for (x,y,w,h) in faces:                 
                self.image = self.desenharRosto(self.image, x, y, w, h)
        else:
            # self.ws.enviarMensagem("Não possui face.")
            if ((current_milli_time() - self.momentoUltimaFaceDetectada) > TEMPO_TOLERANCIA_ROSTO * 1000):
                self.possuiFace = False    
            else:
                if (len(self.oldFaces) > 0):
                    for (x,y,w,h) in self.oldFaces: 
                        self.image = self.desenharRosto(self.image, x, y, w, h, False)


class Person():
    def __init__(self, person_cod, person_name):
        self.person_cod = person_cod
        self.person_name = person_name
        pass

    def get_person_name(self):
        return self.person_name
        
    def get_person_cod(self):
        return self.person_cod


import cv2
from decouple import config

from classes.ws import WebServer
from classes.camera import VideoCamera
from classes.recognize import Recognize

class App():

    def __init__(self):
        camera = VideoCamera()
        web_server = WebServer(camera)
        recognize = Recognize(camera)

        web_server.main()
        recognize.main()


        while(True):
            pass


import time
import threading
import json
import argparse
import requests 

from imutils.video import VideoStream
import face_recognition
import imutils
import pickle
import cv2
from decouple import config

from classes.utils import Utils
from classes.personmanager import PersonManager

DETECTION_METHOD=config('DETECTION_METHOD')
API_ADDRESS = config('API_ADDRESS')

class Recognize():    
    def __init__(self, camera):
        print("[INFO] loading encodings...")
        self.data = pickle.loads(open('face_recognition/encodings.pickle', "rb").read())
        self.camera = camera
        self.start_aprovation_time = 0
        self.names = []
        self.current_name =""
        self.sent_name = ""
        self.clear_names = False
        self.person = {}
        self.too_many_faces_status = False
    def detect(self):        
        # loop over frames from the video file stream
        while True:
            if (self.camera.available and self.camera.mode == 'detect'):
                # grab the frame from the threaded video stream
                frame = self.camera.image
                
                # convert the input frame from BGR to RGB then resize it to have
                # a width of 750px (to speedup processing)
                rgb = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
                rgb = imutils.resize(frame, width=750)
                r = frame.shape[1] / float(rgb.shape[1])

                # detect the (x, y)-coordinates of the bounding boxes
                # corresponding to each face in the input frame, then compute
                # the facial embeddings for each face
                boxes = face_recognition.face_locations(rgb, model=DETECTION_METHOD)
                encodings = face_recognition.face_encodings(rgb, boxes)
                self.names = []

                # loop over the facial embeddings
                for encoding in encodings:
                    # attempt to match each face in the input image to our known
                    # encodings
                    matches = face_recognition.compare_faces(self.data["encodings"], encoding)
                    name = "Unknown"

                    # check to see if we have found a match
                    if True in matches:
                        # find the indexes of all matched faces then initialize a
                        # dictionary to count the total number of times each face
                        # was matched
                        matchedIdxs = [i for (i, b) in enumerate(matches) if b]
                        counts = {}

                        # loop over the matched indexes and maintain a count for
                        # each recognized face face
                        for i in matchedIdxs:
                            name = self.data["names"][i]
                            counts[name] = counts.get(name, 0) + 1

                        # determine the recognized face with the largest number
                        # of votes (note: in the event of an unlikely tie Python
                        # will select first entry in the dictionary)
                        name = max(counts, key=counts.get)
                    
                    # update the list of names
                    self.names.append(name)
                                                     
                if self.too_many_faces_status == True and len(self.names) == 1:
                    self.start_aprovation_time = time.time()
                    self.clear_detected_name()
                    self.too_many_faces_status = False

                if len(self.names) == 1 and self.names[0] != 'Unknown':    
                    self.send_detected_name()
                    self.approve_recognition()
                    
                elif len(self.names) > 1:
                    message = {
                        "action":  "too_many_faces",
                        "status":  True
                    }
                    Utils.send_message_rabbitmq('peekaboo', json.dumps(message))
                    self.too_many_faces_status = True
                    
                elif len(self.names) == 0:
                    self.start_aprovation_time = time.time()
                    self.sent_name = ""
                if len(self.names) != 0:
                    self.clear_names = False 
                elif self.clear_names == False and len(self.names) == 0:
                    self.clear_detected_name()
                

    
    def send_detected_name(self):
        if self.sent_name != self.names[0]:            

            if self.names[0] not in self.person:
                person_name = PersonManager.get_person_name(self.names[0])
                self.person[self.names[0]] = person_name            
            else:
                person_name = self.person[self.names[0]]
            message = {
            "action": "detect",
            "name": person_name,
            "cod": self.names[0]
            }
            Utils.send_message_rabbitmq('peekaboo', json.dumps(message))
            self.sent = True            
            self.sent_name = self.names[0]

    def clear_detected_name(self):        
        message = {
        "action": "clear_detected_name"            
        }
        Utils.send_message_rabbitmq('peekaboo', json.dumps(message))
        self.clear_names = True
            
    
    def approve_recognition(self):     
            if self.names[0] != self.current_name:  
                self.start_aprovation_time = time.time()
                self.current_name = self.names[0]
            else:
                delay_time = time.time() - self.start_aprovation_time
                if delay_time >= 3:

                    url = API_ADDRESS+'baterponto'

                    data = {
                        "CDFUNC": self.names[0]
                    }
                    # sending get request and saving the response as response object 
                    r = requests.post(url = url, data = data) 
                    result = r.json()

                    if result['status']:                        
                        message = {
                            "action": "approve_recognition",
                            "status": True,
                            "cod": self.names[0],
                            "idsentido": result['sentido']
                        }
                        Utils.send_message_rabbitmq('peekaboo', json.dumps(message))                        
                        time.sleep(6)
                        self.clear_detected_name()

                    else:
                        message = {
                            "action": "fail_recognition",
                            "status": True
                        }                        
                        Utils.send_message_rabbitmq('peekaboo', json.dumps(message))  
                        self.clear_detected_name()                                               
                    self.sent_name = ""
                    self.start_aprovation_time = time.time()



    def main(self):    
        t1 = threading.Thread(target=self.detect, daemon=True )
        t1.start()


from imutils import paths
import face_recognition
import argparse
import pickle
import cv2
import os

from decouple import config

DATASET=config('DATASET')
ENCODIGS = config('ENCODIGS')
DETECTION_METHOD=config('DETECTION_METHOD')

class EncodeFace():
	
	def __init__(self):
		# grab the paths to the input images in our dataset
		print("[INFO] quantifying faces...")
		self.imagePaths = list(paths.list_images(DATASET))

		# initialize the list of known encodings and known names
		self.knownEncodings = []
		self.knownNames = []

	def enconde(self):
		# loop over the image paths
		try:
			for (i, imagePath) in enumerate(self.imagePaths):
				# extract the person name from the image path
				print("[INFO] processing image {}/{}".format(i + 1,
					len(self.imagePaths)))
				name = imagePath.split(os.path.sep)[-2]
				print(imagePath)
				# load the input image and convert it from RGB (OpenCV ordering)
				# to dlib ordering (RGB)
				image = cv2.imread(imagePath)
				rgb = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)

				# detect the (x, y)-coordinates of the bounding boxes
				# corresponding to each face in the input image
				boxes = face_recognition.face_locations(rgb,model=DETECTION_METHOD)

				# compute the facial embedding for the face
				encodings = face_recognition.face_encodings(rgb, boxes)

				# loop over the encodings
				for encoding in encodings:
					# add each encoding + name to our set of known names and
					# encodings
					self.knownEncodings.append(encoding)
					self.knownNames.append(name)

			# dump the facial encodings + names to disk
			print("[INFO] serializing encodings...")
			data = {"encodings": self.knownEncodings, "names": self.knownNames}
			f = open(ENCODIGS, "wb")
			f.write(pickle.dumps(data))
			f.close()
			message = {"status": True,
						"message":"Finish"
			}

			return message
		except Exception as e:
			message = {"status": True,
						"message": str(e)
			}			
			print(str(e))
			return message